@@ -3,50 +3,71 @@
 # huge(): Onestep estimation for high-dimensional undirected graph      #
 # Authors: Tuo Zhao and Han Liu                                         #
 # Emails: <tourzhao@andrew.cmu.edu>; <hanliu@cs.jhu.edu>                #
-# Date: Nov 12th 2010                                                   #
-# Version: 0.8.1                                                         #
+# Date: Nov 21st 2010                                                   #
+# Version: 0.9                                                          #
 #-----------------------------------------------------------------------#
 
 ## Main function
-huge = function(L, ind.group = NULL, lambda = NULL, nlambda = NULL, lambda.min.ratio = 0.1, alpha = 1, sym = "or", npn = TRUE, npn.func = "shrinkage", npn.thresh = NULL, approx = FALSE, scr = TRUE, scr.num = NULL, verbose = TRUE){	
+huge = function(L, ind.group = NULL, lambda = NULL, nlambda = NULL, lambda.min.ratio = 0.1, alpha = 1, sym = "or", npn = TRUE, npn.func = "shrinkage", npn.thresh = NULL, method = "GEL", scr = TRUE, scr.num = NULL, verbose = TRUE){	
 	gcinfo(FALSE)
 	est = list()
-	est$approx = approx
+	est$method = method
 	
 	if(is.list(L)){
-		est$data = L$data
 		n = nrow(L$data)
 		d = ncol(L$data)
+		if(d < 3){
+			cat("The fullgraph dimension < 3 and huge() will be teminated....\n")
+			cat("Please refer to Pearson's product-moment correlation....\n")
+			rm(L,alpha,sym,npn.func,method,scr,verbose)
+			gc()
+			est$marker = "Terminated"
+			class(est) = "huge"
+			return(est)
+		}
+		if(!is.null(scr.num))
+			if(scr.num == 1){
+				cat("The neighborhood size < 2 and huge() will be teminated.\n")
+				cat("Please refer to Pearson's product-moment correlation....\n")
+				rm(L,alpha,sym,npn.func,method,scr,verbose)
+				gc()
+				est$marker = "Terminated"
+				class(est) = "huge"
+				return(est)
+		}
+		est$data = L$data
 		if(!is.null(L$theta))	est$theta = L$theta
 	}
+	
 	if(!is.list(L)){
-		est$data = L
 		n = nrow(L)
 		d = ncol(L)
+		if(d < 3){
+			cat("The fullgraph dimension < 3 and huge() will be teminated....\n")
+			cat("Please refer to Pearson's product-moment correlation....\n")
+			rm(L,alpha,sym,npn.func,method,scr,verbose)
+			gc()
+			est$marker = "Terminated"
+			class(est) = "huge"
+			return(est)
+		}
+		if(!is.null(scr.num))
+			if(scr.num == 1){
+				cat("The neighborhood size < 2 and huge() will be teminated.\n")
+				cat("Please refer to Pearson's product-moment correlation....\n")
+				est$marker = "Terminated"
+				rm(L,alpha,sym,npn.func,method,scr,verbose)
+				gc()
+				class(est) = "huge"
+				return(est)
+		}
+		est$data = L
 	}
 	rm(L)
 	gc()	
 	
 	if(is.null(ind.group))	ind.group = c(1:d)
-	k = length(ind.group)
-		
-	if(d < 3){
-		cat("The fullgraph dimension < 3 and huge() will be teminated....\n")
-		cat("Please refer to Pearson's product-moment correlation....\n")
-		est$marker = "Terminated"
-		class(est) = "huge"
-		return(est)
-	}
-	
-	if(!is.null(scr.num))
-		if(scr.num == 1){
-			cat("The neighborhood size < 2 and huge() will be teminated.\n")
-			cat("Please refer to Pearson's product-moment correlation....\n")
-			est$marker = "Terminated"
-			class(est) = "huge"
-			return(est)
-		}
-	
+	k = length(ind.group)	
 	
 	# Nonparanormal transformation
 	if(npn){
@@ -55,9 +76,26 @@ huge = function(L, ind.group = NULL, lambda = NULL, nlambda = NULL, lambda.min.r
 		gc()
 	}
 	
-	if(approx){
-		if(is.null(nlambda)) nlambda = 30
-		fit = huge.scr(est$data, ind.group = ind.group, approx = approx, nlambda = nlambda, lambda.min.ratio = lambda.min.ratio, lambda = lambda, verbose = verbose)
+	if(method == "GLASSO"){
+		if(is.null(lambda)){
+			if(is.null(nlambda)) nlambda = 10
+		}
+		fit = huge.glassoM(est$data, ind.group = ind.group, nlambda = nlambda, lambda.min.ratio = lambda.min.ratio, lambda = lambda, verbose = verbose)
+		est$path = fit$path
+		est$lambda = fit$lambda
+		est$wi = fit$wi
+		est$df = fit$df
+		est$sparsity = fit$sparsity
+		est$loglik = fit$loglik
+		rm(fit)
+		gc()
+	}			
+			
+	if(method == "GACT"){
+		if(is.null(lambda)){
+			if(is.null(nlambda)) nlambda = 30
+		}
+		fit = huge.scr(est$data, ind.group = ind.group, method = method, nlambda = nlambda, lambda.min.ratio = lambda.min.ratio, lambda = lambda, verbose = verbose)
 		est$path = fit$path
 		est$lambda = fit$lambda
 		est$sparsity = fit$sparsity
@@ -65,12 +103,11 @@ huge = function(L, ind.group = NULL, lambda = NULL, nlambda = NULL, lambda.min.r
 		gc()
 	}
 	
-	if(!approx){		
+	if(method == "GEL"){		
 		if(!scr){
 			ind.mat = NULL
-			est$scr = scr
+			est$scr = FALSE
 		}
-		
 		if(scr){
 			if(is.null(scr.num)&&(n >= d)){
 				ind.mat = NULL
@@ -86,15 +123,17 @@ huge = function(L, ind.group = NULL, lambda = NULL, nlambda = NULL, lambda.min.r
 				if(scr.num >= (d-1)){
 					if(verbose) cat("The specified scr.num >= (d-1) and Graph SURE Screening will be skipped.\n")
 					ind.mat = NULL
+					est$scr = FALSE
 				}
 				if(scr.num < (d-1)){
 					ind.mat = huge.scr(est$data, ind.group = ind.group, scr.num = scr.num, verbose = verbose)$ind.mat
-					est$scr = scr
+					est$scr = TRUE
 				}
 			}
 		}
-		if(is.null(nlambda)) nlambda = 10
-		
+		if(is.null(lambda)){
+			if(is.null(nlambda)) nlambda = 10
+		}
 		fit = huge.subgraph(est$data, ind.group = ind.group, ind.mat = ind.mat, alpha = alpha, lambda = lambda, nlambda = nlambda, lambda.min.ratio = lambda.min.ratio, sym = sym, verbose = verbose)
 		est$path = fit$path
 		est$lambda = fit$lambda
@@ -107,7 +146,7 @@ huge = function(L, ind.group = NULL, lambda = NULL, nlambda = NULL, lambda.min.r
 		est$ind.mat = ind.mat
 		est$alpha = alpha
 		est$sym = sym
-		rm(ind.mat,alpha,sym,approx)
+		rm(ind.mat,alpha,sym,method)
 		gc()
 	}
 	est$ind.group = ind.group
@@ -115,7 +154,7 @@ huge = function(L, ind.group = NULL, lambda = NULL, nlambda = NULL, lambda.min.r
 	if(k==d) est$type = "Fullgraph solution path"
 	est$npn = npn
 	
-	rm(ind.group,n,k,d,npn,scr,lambda,lambda.min.ratio,nlambda)
+	rm(ind.group,n,k,d,npn,scr,lambda,lambda.min.ratio,nlambda,verbose)
 	gc()	
 	
 	est$marker = "Successful"
@@ -131,16 +170,16 @@ print.huge = function(x, ...){
 		return("Please refer to the manual")
 	}
 	
-	if(x$approx) cat("Model: Graph Approximation via Correlation Thresholding (GACT) --->",x$type,"\n")
-	
-	if(!x$approx){
+	if(x$method == "GACT") cat("Model: Graph Approximation via Correlation Thresholding (GACT) --->",x$type,"\n")
+	if(x$method == "GLASSO") cat("Model: Graphical Lasso (GLASSO) --->",x$type,"\n")
+	if(x$method == "GEL"){
 		if(x$alpha <1) cat("Model: Meinshausen &Buhlmann Graph Estimation via Elastic Net --->",x$type,"\n")
 		if(x$alpha ==1) cat("Model: Meinshausen &Buhlmann Graph Estimation via Lasso (GEL) --->",x$type,"\n")
 	}
 	
 	if(x$npn) cat("Nonparanormal (NPN) transformation: on\n")
 	
-	if(!x$approx&&x$scr) cat("Graph SURE Screening (GSS): on\n")
+	if((x$method == "GEL")&&x$scr) cat("Graph SURE Screening (GSS): on\n")
 
 	if(is.null(x$theta)) cat("True graph: not included\n")
 	if(!is.null(x$theta)) cat("True graph: included\n")
@@ -157,15 +196,15 @@ summary.huge = function(object, ...){
 		cat("huge() has been terminated\n")
 		return("Please refer to the manual")
 	}
-	if(object$approx) cat("Model: Graph Approximation via Correlation Thresholding (GACT) --->",object$type,"\n")
-	
-	if(!object$approx){
+	if(object$method == "GACT") cat("Model: Graph Approximation via Correlation Thresholding (GACT) --->",object$type,"\n")
+	if(object$method == "GLASSO") cat("Model: Graphical Lasso (GLASSO) --->",object$type,"\n")
+	if(object$method == "GEL"){
 		if(object$alpha <1) cat("Model: Meinshausen &Buhlmann Graph Estimation via Elastic Net --->",object$type,"\n")
 		if(object$alpha ==1) cat("Model: Meinshausen &Buhlmann Graph Estimation via Lasso (GEL)--->",object$type,"\n")
 	}
 	
 	if(object$npn) cat("Nonparanormal (NPN) transformation: on\n")
-	if(!object$approx&&object$scr) cat("Graph SURE Screening (GSS): on\n")
+	if((object$method == "GEL")&&object$scr) cat("Graph SURE Screening (GSS): on\n")
 
 	if(is.null(object$theta)) cat("True graph: not included\n")
 	if(!is.null(object$theta)) cat("True graph: included\n")
@@ -187,8 +226,6 @@ plot.huge = function(x, align = FALSE, ...){
 	if(length(x$lambda) == 2)	par(mfrow = c(1, 3), pty = "s", omi=c(0.3,0.3,0.3,0.3), mai = c(0.3,0.3,0.3,0.3))
 	if(length(x$lambda) >= 3)	par(mfrow = c(2, 2), pty = "s", omi=c(0.3,0.3,0.3,0.3), mai = c(0.3,0.3,0.3,0.3))
 	
-	
-	
 	if(length(x$lambda) <= 3)	z.final = 1:length(x$lambda)
 	
 	if(length(x$lambda) >=4){
@@ -196,6 +233,7 @@ plot.huge = function(x, align = FALSE, ...){
 		z.min = min(x$sparsity)
 		z = z.max - z.min
 		z.unique = unique(c(which(x$sparsity>=(z.min + 0.03*z))[1],which(x$sparsity>=(z.min + 0.07*z))[1],which(x$sparsity>=(z.min + 0.15*z))[1]))
+
 		
 		if(length(z.unique) == 1){
 			if(z.unique<(length(x$lambda)-1))	z.final = c(z.unique,z.unique+1,z.unique+2)
@@ -213,29 +251,32 @@ plot.huge = function(x, align = FALSE, ...){
 		
 		if(length(z.unique) == 3) z.final = z.unique
 		
-		rm(z.max,z.min,z)
+		rm(z.max,z.min,z,z.unique)
 		gc()
+		
 	}
+	plot(x$lambda, x$sparsity, log = "x", xlab = "Regularization Parameter", ylab = "Sparsity Level", type = "l",xlim = rev(range(x$lambda)), main = "Sparsity vs. Regularization")
 	
-	plot(x$lambda, x$sparsity, log = "x", xlab = "Regularization Parameters", ylab = "Sparsity Level", type = "l",xlim = rev(range(x$lambda)), main = "Sparsity vs. Regularization")
-	
-	lines(x$lambda[c(z.final)],x$sparsity[c(z.final)],type = "p")
+	lines(x$lambda[z.final],x$sparsity[z.final],type = "p")
 	
 	if(align){
 		layout.grid = layout.fruchterman.reingold(graph.adjacency(as.matrix(x$path[[z.final[length(z.final)]]]), mode="undirected", diag=FALSE))
 		for(i in z.final){
 			g = graph.adjacency(as.matrix(x$path[[i]]), mode="undirected", diag=FALSE)
 			plot(g, layout=layout.grid, edge.color='gray50',vertex.color="red", vertex.size=5, vertex.label=NA, main = paste("lambda = ",as.character(round(x$lambda[i],3)),sep = ""))
+			rm(g)
+			gc()
 		}
+	rm(layout.grid)
 	}
 	if(!align){
 		for(i in z.final){
 			g = graph.adjacency(as.matrix(x$path[[i]]), mode="undirected", diag=FALSE)
 			layout.grid = layout.fruchterman.reingold(g)
 			plot(g, layout=layout.grid, edge.color='gray50',vertex.color="red", vertex.size=5, vertex.label=NA, main = paste("lambda = ",as.character(round(x$lambda[i],3)),sep = ""))
+			rm(g,layout.grid)
+			gc()
 		}
 	}
-	rm(g,layout.grid)
-	gc()
-	if(!align) cat("Three plotted graphs are aligned according to the third graph\n")
+	if(align) cat("Three plotted graphs are aligned according to the third graph\n")
 }
\ No newline at end of file
